@@ -2,7 +2,7 @@
 
     Author:        Jan Wielemaker and Anjo Anjewierden
     E-mail:        J.Wielemaker@vu.nl
-    WWW:           http://www.swi-prolog.org/projects/xpce/
+    WWW:           http://www.swi-prolog.org/packages/xpce/
     Copyright (c)  2001-2016, University of Amsterdam
                               VU University Amsterdam
     All rights reserved.
@@ -80,23 +80,27 @@ user:prolog_trace_interception(Port, Frame, CHP, Action) :-
     with_access_user(prolog_trace_interception_gui(Port, Frame, CHP, Action)).
 
 prolog_trace_interception_gui(Port, Frame, CHP, Action) :-
+    State = state(0),
     current_prolog_flag(gui_tracer, true),
-    (   notrace(intercept(Port, Frame, CHP, GuiAction)),
-        map_action(GuiAction, Frame, Action)
-    ->  true
-    ;   print_message(warning,
-                      guitracer(intercept_failed(Port, Frame,
-                                                 CHP, Action))),
-        Action = continue
+    (   (   '$notrace'(intercept(Port, Frame, CHP, GuiAction)),
+            map_action(GuiAction, Frame, Action)
+        ->  true
+        ;   print_message(warning,
+                          guitracer(intercept_failed(Port, Frame,
+                                                     CHP, Action))),
+            Action = continue
+        ),
+        nb_setarg(1, State, Action),
+        fail
+    ;   arg(1, State, Action)
     ).
 
-:- multifile
-    user:prolog_event_hook/1.
-
-user:prolog_event_hook(frame_finished(Frame)) :-
-    retractall(finished_frame(Frame)),
-    fail.
+:- initialization
+    prolog_unlisten(frame_finished, retract_frame),
+    prolog_listen(frame_finished, retract_frame).
 
+retract_frame(Frame) :-
+    retractall(finished_frame(Frame)).
 
 %!  map_action(+GuiAction, +Frame, -Action) is det.
 %
@@ -550,6 +554,7 @@ clause_end(ClauseRef, File, CharA, CharZ) :-
     pce_clause_info(ClauseRef, File, TPos, _),
     nonvar(TPos),
     arg(2, TPos, CharA),
+    nonvar(CharA),
     CharZ is CharA + 1.
 
 head_pos(Ref, Pos, HPos) :-
