@@ -3,7 +3,7 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2011-2022, University of Amsterdam
+    Copyright (c)  2011-2023, University of Amsterdam
                               VU University Amsterdam
 			      CWI, Amsterdam
 			      SWI-Prolog Solutions b.v.
@@ -60,27 +60,6 @@
 #undef LD
 #define LD LOCAL_LD
 
-/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-General file operations and binding to Prolog
-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
-
-#ifdef __WINDOWS__
-static void
-set_posix_error(int win_error)
-{ int error = 0;
-
-  switch(win_error)
-  { case ERROR_ACCESS_DENIED:	  error = EACCES; break;
-    case ERROR_FILE_NOT_FOUND:    error = ENOENT; break;
-    case ERROR_SHARING_VIOLATION: error = EAGAIN; break;
-    case ERROR_ALREADY_EXISTS:    error = EEXIST; break;
-  }
-
-  errno = error;
-}
-#endif /*__WINDOWS__*/
-
-
 		 /*******************************
 		 *	      OS STUFF		*
 		 *******************************/
@@ -97,46 +76,8 @@ intervals since January 1, 1601 (UTC).
 int
 LastModifiedFile(const char *name, double *tp)
 {
-#ifdef __WINDOWS__
-  HANDLE hFile;
-  wchar_t wfile[PATH_MAX];
-
-#define nano * 0.000000001
-#define ntick 100.0
-#define SEC_TO_UNIX_EPOCH 11644473600.0
-
-  if ( !_xos_os_filenameW(name, wfile, PATH_MAX) )
-    return FALSE;
-
-  if ( (hFile=CreateFileW(wfile,
-			  0,
-			  FILE_SHARE_DELETE|FILE_SHARE_READ|FILE_SHARE_WRITE,
-			  NULL,
-			  OPEN_EXISTING,
-			  FILE_FLAG_BACKUP_SEMANTICS,
-			  NULL)) != INVALID_HANDLE_VALUE )
-  { FILETIME wt;
-    int rc;
-
-    rc = GetFileTime(hFile, NULL, NULL, &wt);
-    CloseHandle(hFile);
-
-    if ( rc )
-    { double t;
-
-      t  = (double)wt.dwHighDateTime * (4294967296.0 * ntick nano);
-      t += (double)wt.dwLowDateTime  * (ntick nano);
-      t -= SEC_TO_UNIX_EPOCH;
-
-      *tp = t;
-
-      return TRUE;
-    }
-  }
-
-  set_posix_error(GetLastError());
-
-  return FALSE;
+#ifdef O_XOS
+  return _xos_get_file_time(name, XOS_TIME_MODIFIED, tp) == 0;
 #else
   char tmp[PATH_MAX];
   statstruct buf;
@@ -163,13 +104,23 @@ be accessed.
 
 static int64_t
 SizeFile(const char *path)
-{ char tmp[PATH_MAX];
+{
+#if O_XOS
+  uint64_t size;
+
+  if ( _xos_file_size(path, &size) == 0 )
+    return (int64_t)size;
+
+  return -1;
+#else
+  char tmp[PATH_MAX];
   statstruct buf;
 
   if ( statfunc(OsPath(path, tmp), &buf) < 0 )
     return -1;
 
   return buf.st_size;
+#endif
 }
 
 
@@ -227,7 +178,7 @@ int
 ExistsFile(const char *path)
 {
 #ifdef O_XOS
-  return _xos_exists(path, _XOS_FILE);
+  return _xos_exists(path, _XOS_FILE) == TRUE;
 #else
   char tmp[PATH_MAX];
   statstruct buf;
@@ -245,7 +196,7 @@ int
 ExistsDirectory(const char *path)
 {
 #ifdef O_XOS
-  return _xos_exists(path, _XOS_DIR);
+  return _xos_exists(path, _XOS_DIR) == TRUE;
 #else
   char tmp[PATH_MAX];
   char *ospath = OsPath(path, tmp);
@@ -279,7 +230,7 @@ ReadLink(const char *f, char *buf)
 
 
 static char *
-DeRefLink1(const char *f, char *lbuf)
+DeRefLink1(const char *f, char *lbuf, size_t buflen)
 { char buf[PATH_MAX];
   char *l;
 
@@ -297,9 +248,7 @@ DeRefLink1(const char *f, char *lbuf)
 	q--;
       strcpy(q, l);
 
-      canonicaliseFileName(lbuf);
-
-      return lbuf;
+      return canonicaliseFileName(lbuf, buflen);
     }
   }
 
@@ -320,8 +269,10 @@ DeRefLink(const	char *link, char *buf)
   char *f;
   int n = 20;				/* avoid loop! */
 
-  while((f=DeRefLink1(link, tmp)) && n-- > 0)
+  while((f=DeRefLink1(link, tmp, sizeof(tmp))) && n-- > 0)
     link = f;
+  if ( PL_exception(0) )		/* Name too long */
+    return NULL;
 
   if ( n > 0 )
   { strcpy(buf, link);
@@ -371,7 +322,7 @@ SameFile(const char *f1, const char *f2)
   }
 #endif
 #ifdef O_XOS
-  return _xos_same_file(f1, f2);
+  return _xos_same_file(f1, f2) == TRUE;
 #endif /*O_XOS*/
     /* Amazing! There is no simple way to check two files for identity. */
     /* stat() and fstat() both return dummy values for inode and device. */
@@ -430,7 +381,8 @@ RenameFile(const char *old, const char *new)
 static int
 MarkExecutable(const char *name)
 {
-#if (defined(HAVE_STAT) && defined(HAVE_CHMOD)) || defined(__unix__)
+#if ( (defined(HAVE_STAT) && defined(HAVE_CHMOD)) || \
+      defined(__unix__) ) && !defined(__WINDOWS__)
   statstruct buf;
   mode_t um;
 
@@ -441,7 +393,7 @@ MarkExecutable(const char *name)
     term_t file = PL_new_term_ref();
 
     PL_put_atom_chars(file, name);
-    return PL_error(NULL, 0, OsError(), ERR_FILE_OPERATION,
+    return PL_error(NULL, 0, MSG_ERRNO, ERR_FILE_OPERATION,
 		    ATOM_stat, ATOM_file, file);
   }
 
@@ -454,7 +406,7 @@ MarkExecutable(const char *name)
     term_t file = PL_new_term_ref();
 
     PL_put_atom_chars(file, name);
-    return PL_error(NULL, 0, OsError(), ERR_FILE_OPERATION,
+    return PL_error(NULL, 0, MSG_ERRNO, ERR_FILE_OPERATION,
 		    ATOM_chmod, ATOM_file, file);
   }
 #endif /* defined(HAVE_STAT) && defined(HAVE_CHMOD) */
@@ -469,7 +421,8 @@ MarkExecutable(const char *name)
 
 int
 unifyTime(term_t t, time_t time)
-{ return PL_unify_time(t, time);
+{ GET_LD
+  return PL_unify_time(t, time);
 }
 
 
@@ -491,7 +444,7 @@ add_option(term_t options, functor_t f, atom_t val)
 #define CVT_FILENAME (CVT_ATOM|CVT_STRING|CVT_LIST)
 
 static int
-get_file_name(term_t n, char **namep, char *tmp, int flags)
+get_file_name(term_t n, char **namep, char *tmp, size_t tmplen, int flags)
 { GET_LD
   char *name;
   int chflags;
@@ -578,7 +531,7 @@ get_file_name(term_t n, char **namep, char *tmp, int flags)
   }
 
   if ( flags & PL_FILE_ABSOLUTE )
-  { if ( !(name = AbsoluteFile(name, tmp)) )
+  { if ( !(name = AbsoluteFile(name, tmp, tmplen)) )
       return FALSE;
   }
 
@@ -595,7 +548,7 @@ PL_get_file_name(term_t n, char **namep, int flags)
   char *name;
   int rc;
 
-  if ( (rc=get_file_name(n, &name, buf, flags)) )
+  if ( (rc=get_file_name(n, &name, buf, sizeof(buf), flags)) )
   { if ( (flags & PL_FILE_OSPATH) )
     { if ( !(name = OsPath(name, ospath)) )
 	return FALSE;
@@ -616,7 +569,7 @@ PL_get_file_nameW(term_t n, wchar_t **namep, int flags)
   char *name;
   int rc;
 
-  if ( (rc=get_file_name(n, &name, buf, flags|REP_UTF8)) )
+  if ( (rc=get_file_name(n, &name, buf, sizeof(buf), flags|REP_UTF8)) )
   { Buffer b;
     const char *s;
 
@@ -684,7 +637,7 @@ PRED_IMPL("size_file", 2, size_file, 0)
     }
 
     if ( (size = SizeFile(n)) < 0 )
-      return PL_error("size_file", 2, OsError(), ERR_FILE_OPERATION,
+      return PL_error("size_file", 2, MSG_ERRNO, ERR_FILE_OPERATION,
 		      ATOM_size, ATOM_file, A1);
 
     return PL_unify_int64(A2, size);
@@ -1018,7 +971,7 @@ PRED_IMPL("rename_file", 2, rename_file, 0)
       return TRUE;
 
     if ( truePrologFlag(PLFLAG_FILEERRORS) )
-      return PL_error("rename_file", 2, OsError(), ERR_FILE_OPERATION,
+      return PL_error("rename_file", 2, MSG_ERRNO, ERR_FILE_OPERATION,
 		      ATOM_rename, ATOM_file, old);
     return FALSE;
   }
@@ -1036,7 +989,7 @@ PRED_IMPL("$absolute_file_name", 2, absolute_file_name, 0)
   term_t expanded = A2;
 
   if ( PL_get_file_name(name, &n, 0) )
-  { if ( (n = AbsoluteFile(n, tmp)) )
+  { if ( (n = AbsoluteFile(n, tmp, sizeof(tmp))) )
       return PL_unify_chars(expanded, PL_ATOM|REP_FN, -1, n);
   }
 
@@ -1174,13 +1127,13 @@ PRED_IMPL("prolog_to_os_filename", 2, prolog_to_os_filename, 0)
 
   if ( !PL_is_variable(pl) )
   { char *n;
-    wchar_t buf[PATH_MAX];
+    char buf[PATH_MAX];
 
     if ( PL_get_chars(pl, &n, CVT_ALL|REP_UTF8|CVT_EXCEPTION) )
-    { if ( !_xos_os_filenameW(n, buf, PATH_MAX) )
+    { if ( !_xos_os_filename(n, buf, PATH_MAX) )
 	return name_too_long();
 
-      return PL_unify_wchars(os, PL_ATOM, -1, buf);
+      return PL_unify_chars(os, PL_ATOM|REP_UTF8, -1, buf);
     }
     return FALSE;
   }
