@@ -3,8 +3,9 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2011-2020, University of Amsterdam
+    Copyright (c)  2011-2024, University of Amsterdam
 			      CWI, Amsterdam
+			      SWI-Prolog Solutions b.v.
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -105,6 +106,71 @@ variables.
 #define COPY_MARKED	0x08			/* Only copy marked variables */
 #define COPYING_ATTVAR  0x10			/* See mark_for_copy() */
 
+/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+mark_vars() is a helper for copy_term/4.
+- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
+
+#define mark_vars(t, set) LDFUNC(mark_vars, t, set)
+
+static int
+mark_vars(DECL_LD term_t t, int set)
+{ int rc = TRUE;
+  term_agenda agenda;
+  Word p = valTermRef(t);
+
+  initTermAgenda(&agenda, 1, p);
+  while((p=nextTermAgenda(&agenda)))
+  { switch(tag(*p))
+    { case TAG_ATTVAR:
+      case TAG_VAR:
+	if ( set )
+	  set_copy(*p);
+        else
+	  clear_marks(*p);
+        break;
+      case TAG_COMPOUND:
+      {	if ( !visited(*p) )
+	{ Functor t = valueTerm(*p);
+	  int arity = arityFunctor(t->definition);
+
+	  set_visited(*p);
+	  if ( !pushWorkAgenda(&agenda, arity, t->arguments) )
+	  { rc = MEMORY_OVERFLOW;
+	    goto end_loop;
+	  }
+	}
+      }
+    }
+  }
+end_loop:
+  clearTermAgenda(&agenda);
+
+  p = valTermRef(t);
+  initTermAgenda(&agenda, 1, p);
+  while((p=nextTermAgenda(&agenda)))
+  { switch(tag(*p))
+    { case TAG_ATTVAR:
+      case TAG_VAR:
+	if ( rc != TRUE )
+	  clear_marks(*p);
+        break;
+      case TAG_COMPOUND:
+      { if ( visited(*p) )
+	{ clear_marks(*p);
+
+	  Functor t = valueTerm(*p);
+	  int arity = arityFunctor(t->definition);
+	  if ( !pushWorkAgenda(&agenda, arity, t->arguments) )
+	    fatalError("No memory for cleaning\n");
+	}
+      }
+    }
+  }
+
+  return rc;
+}
+
+
 #define mark_for_duplicate(p, flags) LDFUNC(mark_for_duplicate, p, flags)
 static int
 mark_for_duplicate(DECL_LD Word p, int flags)
@@ -240,20 +306,24 @@ update_ground(DECL_LD Word p, int flags)
 #define MC_WALK_REF  0x1
 #define MC_WALK_COPY 0x2
 
+#define MC_TAG(p,t)  ((Word)((uintptr_t)(p)|t))
+#define MC_UNTAG(p)  ((Word)((uintptr_t)(p) & ~(uintptr_t)MC_MASK))
+#define MC_TAGVAL(p) ((int)((uintptr_t)(p) & (uintptr_t)MC_MASK))
+
 static int
 pushForMark(segstack *stack, Word p, int wr)
-{ word w = ((word)p)|wr;
+{ Word w = MC_TAG(p, wr);
 
-  return pushSegStack(stack, w, word);
+  return pushSegStack(stack, w, Word);
 }
 
 static void
 popForMark(segstack *stack, Word *pp, int *wr)
-{ word w = 0;
+{ Word w = 0;
 
-  popSegStack(stack, &w, word);
-  *wr = w & (word)MC_MASK;
-  *pp = (Word)(w & ~(word)MC_MASK);
+  popSegStack(stack, &w, Word);
+  *wr = MC_TAGVAL(w);
+  *pp = MC_UNTAG(w);
 }
 
 
@@ -341,7 +411,7 @@ mark_for_copy(DECL_LD Word p, int flags)
 	if ( virgin(t->definition) )
 	{ DEBUG(MSG_COPYTERM,
 		Sdprintf("Visit %s at %s\n",
-			 functorName(t->definition),
+			 functorName(word2functor(t->definition)),
 			 print_addr(&t->definition, NULL)));
 	  set_visited(t->definition);
 	} else
@@ -386,7 +456,7 @@ mark_for_copy(DECL_LD Word p, int flags)
 
       DEBUG(MSG_COPYTERM,
 	    Sdprintf("Functor %s; mode=0x%x\n",
-		     functorName(*p & ~BOTH_MASK), mode));
+		     functorName(word2functor(*p & ~BOTH_MASK)), mode));
 
       if ( mode&MC_WALK_COPY )
 	f |= COPYING_ATTVAR;
@@ -433,7 +503,7 @@ cp_unmark(DECL_LD Word p, int flags)
 
 	  DEBUG(MSG_COPYTERM,
 		Sdprintf("Unmarking %s at %s\n",
-			 functorName(f->definition),
+			 functorName(word2functor(f->definition)),
 			 print_addr(&f->definition, NULL)));
 
 	  if ( !pushWorkAgenda(&agenda,
@@ -636,7 +706,7 @@ copy_term(DECL_LD Word from, Word to, size_t abstract, int flags)
 	}
       }
       case TAG_ATOM:
-	pushVolatileAtom(*from);
+	pushVolatileAtom(word2atom(*from));
         /*FALLTHROUGH*/
       default:
 	*to = *from;
@@ -673,7 +743,7 @@ again:
     case TAG_COMPOUND:
       break;
     case TAG_ATOM:
-      pushVolatileAtom(*from);
+      pushVolatileAtom(word2atom(*from));
       /*FALLTHROUGH*/
     default:
       *to = *from;
@@ -709,9 +779,12 @@ again:
 }
 
 
-#define copy_term_refs(from, to, abstract, flags) LDFUNC(copy_term_refs, from, to, abstract, flags)
+#define copy_term_refs(from, to, vars, abstract, flags) \
+	LDFUNC(copy_term_refs, from, to, vars, abstract, flags)
+
 static int
-copy_term_refs(DECL_LD term_t from, term_t to, size_t abstract, int flags)
+copy_term_refs(DECL_LD term_t from, term_t to, term_t vars,
+	       size_t abstract, int flags)
 { for(;;)
   { fid_t fid;
     int rc;
@@ -728,13 +801,22 @@ copy_term_refs(DECL_LD term_t from, term_t to, size_t abstract, int flags)
     *valTermRef(to) = makeRefG(dest);
     src = valTermRef(from);
 
+    if ( vars )
+    { if ( mark_vars(vars, TRUE) != TRUE )
+	return PL_no_memory();
+    }
     rc = do_copy_term(src, dest, abstract, flags);
 
     if ( rc < 0 )			/* no space for copy */
     { PL_discard_foreign_frame(fid);
       PL_put_variable(to);		/* gc consistency */
+      if ( vars )
+      { if ( mark_vars(vars, FALSE) != TRUE )
+	  return PL_no_memory();
+      }
       if ( !makeMoreStackSpace(rc, ALLOW_SHIFT|ALLOW_GC) )
 	return FALSE;
+      DEBUG(CHK_SECURE, checkStacks(NULL));
     } else
     { PL_close_foreign_frame(fid);
       DEBUG(CHK_SECURE,
@@ -750,41 +832,13 @@ copy_term_refs(DECL_LD term_t from, term_t to, size_t abstract, int flags)
 
 int
 duplicate_term(DECL_LD term_t in, term_t copy)
-{ return copy_term_refs(in, copy, (size_t)-1, COPY_ATTRS);
+{ return copy_term_refs(in, copy, 0, (size_t)-1, COPY_ATTRS);
 }
 
 
 int
 size_abstract_term(DECL_LD term_t in, term_t copy, size_t abstract)
-{ return copy_term_refs(in, copy, abstract, COPY_ATTRS|COPY_ABSTRACT);
-}
-
-#define mark_vars(t) LDFUNC(mark_vars, t)
-
-static int
-mark_vars(DECL_LD term_t t)
-{ term_agenda agenda;
-  Word p = valTermRef(t);
-
-  initTermAgenda(&agenda, 1, p);
-  while((p=nextTermAgenda(&agenda)))
-  { switch(tag(*p))
-    { case TAG_ATTVAR:
-      case TAG_VAR:
-	set_copy(*p);
-        break;
-      case TAG_COMPOUND:
-      { Functor t = valueTerm(*p);
-	int arity = arityFunctor(t->definition);
-
-	if ( !pushWorkAgenda(&agenda, arity, t->arguments) )
-	  return MEMORY_OVERFLOW;
-      }
-    }
-  }
-  clearTermAgenda(&agenda);
-
-  return TRUE;
+{ return copy_term_refs(in, copy, 0, abstract, COPY_ATTRS|COPY_ABSTRACT);
 }
 
 
@@ -792,6 +846,29 @@ mark_vars(DECL_LD term_t t)
 		 *	  FAST HEAP TERMS	*
 		 *******************************/
 
+/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+This Prolog binding is for testing purposes only.
+- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
+
+static
+PRED_IMPL("$fast_record", 2, fast_record, 0)
+{ PRED_LD
+  fastheap_term *t = term_to_fastheap(A1);
+  return PL_unify_pointer(A2, t);
+}
+
+static
+PRED_IMPL("$fast_recorded", 2, fast_recorded, 0)
+{ PRED_LD
+  void *ptr;
+
+  if ( !PL_get_pointer_ex(A1, &ptr) )
+    return FALSE;
+
+  term_t tmp = PL_new_term_ref();
+  return put_fastheap(ptr, tmp) && PL_unify(A2, tmp);
+}
+
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 The code below copies a  term  to   the  heap  (program space) just like
 PL_record(). The representation is particularly   suited  for copying it
@@ -809,16 +886,12 @@ needs_relocation(word w)
 { return ( isTerm(w) || isRef(w) || isIndirect(w) || isAtom(w) );
 }
 
-#if SIZEOF_VOIDP == 8
-#define PTR_SHIFT (LMASK_BITS+1)
-#else
 #define PTR_SHIFT LMASK_BITS
-#endif
 
 static word
 relocate_down(word w, size_t offset)
 { if ( isAtom(w) )
-  { PL_register_atom(w);
+  { PL_register_atom(word2atom(w));
     return w;
   } else
   { return (((w>>PTR_SHIFT)-offset)<<PTR_SHIFT) | tagex(w);
@@ -829,7 +902,7 @@ relocate_down(word w, size_t offset)
 static word
 relocate_up(DECL_LD word w, size_t offset)
 { if ( isAtom(w) )
-  { pushVolatileAtom(w);
+  { pushVolatileAtom(word2atom(w));
     return w;
   } else
   { return (((w>>PTR_SHIFT)+offset)<<PTR_SHIFT) | tagex(w);
@@ -878,7 +951,7 @@ term_to_fastheap(DECL_LD term_t t)
   fht->relocations = addPointer(fht->data, fht->data_len*sizeof(word));
   indirects        = fht->data + (gtop-gcopy);
 
-  offset = gcopy-gBase;
+  offset = (size_t)gcopy;
   for(p=gcopy, o=fht->data, r=fht->relocations; p<gtop; p++)
   { if ( needs_relocation(*p) )
     { size_t this_rel = p-gcopy;
@@ -886,10 +959,9 @@ term_to_fastheap(DECL_LD term_t t)
       if ( isIndirect(*p) )
       { Word ip = addressIndirect(*p);
 	size_t sz = wsizeofInd(*ip)+2;
-	size_t go = gBase - (Word)base_addresses[STG_GLOBAL];
 
 	memcpy(indirects, ip, sz*sizeof(word));
-	*o++ = ((go+indirects-fht->data)<<PTR_SHIFT) | tagex(*p);
+	*o++ = (((char*)indirects-(char*)fht->data)<<PTR_SHIFT) | tagex(*p);
 	indirects += sz;
       } else
       { *o++ = relocate_down(*p, offset);
@@ -914,7 +986,7 @@ free_fastheap(fastheap_term *fht)
   for(r = fht->relocations; *r != REL_END; r++)
   { p += *r;
     if ( isAtom(*p) )
-      PL_unregister_atom(*p);
+      PL_unregister_atom(word2atom(*p));
   }
 
   free(fht);
@@ -937,7 +1009,7 @@ put_fastheap(DECL_LD fastheap_term *fht, term_t t)
   o = gTop;
   memcpy(o, fht->data, fht->data_len*sizeof(word));
 
-  offset = o-gBase;
+  offset = (size_t)o;
   for(r = fht->relocations, p=o; *r != REL_END; r++)
   { p += *r;
     *p = relocate_up(*p, offset);
@@ -945,6 +1017,7 @@ put_fastheap(DECL_LD fastheap_term *fht, term_t t)
 
   gTop += fht->data_len;
   *valTermRef(t) = makeRefG(o);
+  DEBUG(CHK_SECURE, PL_check_data(t));
 
   return TRUE;
 }
@@ -963,7 +1036,7 @@ PRED_IMPL("copy_term", 2, copy_term, 0)
   } else
   { term_t copy = PL_new_term_ref();
 
-    if ( copy_term_refs(A1, copy, (size_t)-1, COPY_SHARE|COPY_ATTRS) )
+    if ( copy_term_refs(A1, copy, 0, (size_t)-1, COPY_SHARE|COPY_ATTRS) )
       return PL_unify(copy, A2);
 
     fail;
@@ -981,8 +1054,7 @@ copy_term_4(DECL_LD term_t vs0, term_t t0, term_t vs, term_t t, int flags)
   return ( (term = PL_new_term_ref()) &&
 	   (copy = PL_new_term_ref()) &&
 	   PL_cons_functor(term, FUNCTOR_minus2, vs0, t0) &&
-	   mark_vars(vs0) &&
-	   copy_term_refs(term, copy, (size_t)-1,
+	   copy_term_refs(term, copy, vs0, (size_t)-1,
 			  COPY_SHARE|COPY_MARKED|flags) &&
 	   PL_get_arg(1, copy, term) &&
 	   PL_unify(vs, term) &&
@@ -1023,7 +1095,7 @@ PRED_IMPL("copy_term_nat", 2, copy_term_nat, 0)
 { PRED_LD
   term_t copy = PL_new_term_ref();
 
-  if ( copy_term_refs(A1, copy, (size_t)-1, COPY_SHARE) )
+  if ( copy_term_refs(A1, copy, 0, (size_t)-1, COPY_SHARE) )
     return PL_unify(copy, A2);
 
   fail;
@@ -1056,4 +1128,7 @@ BeginPredDefs(copyterm)
   PRED_DEF("copy_term_nat",      2, copy_term_nat,      0)
   PRED_DEF("copy_term_nat",      4, copy_term_nat,      0)
   PRED_DEF("size_abstract_term", 3, size_abstract_term, 0)
+/* Predicates for testing */
+  PRED_DEF("$fast_record",       2, fast_record,        0)
+  PRED_DEF("$fast_recorded",     2, fast_recorded,      0)
 EndPredDefs
