@@ -3,7 +3,7 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2008-2023, University of Amsterdam
+    Copyright (c)  2008-2024, University of Amsterdam
 			      CWI, Amsterdam
 			      SWI-Prolog Solutions b.v.
     All rights reserved.
@@ -72,7 +72,7 @@
 */
 
 #define HAVE_MSB 1
-static inline int
+static inline unsigned int
 MSB(size_t i)
 { unsigned long index;
 #if SIZEOF_VOIDP == 8
@@ -88,7 +88,7 @@ MSB(size_t i)
 
 #if SIZEOF_VOIDP == 8
 #define HAVE_MSB64 1
-static inline int
+static inline unsigned int
 MSB64(int64_t i)
 { unsigned long index;
   _BitScanReverse64(&index, i);
@@ -124,15 +124,25 @@ __builtin_saddll_overflow(long long int a, long long int b, long long int *res)
 #endif /*_MSC_VER*/
 
 #if !defined(HAVE_MSB) && defined(HAVE__BUILTIN_CLZ)
-#if SIZEOF_VOIDP == SIZEOF_LONG
-#define MSB(i) ((int)sizeof(long)*8-1-__builtin_clzl(i)) /* GCC builtin */
 #define HAVE_MSB 1
-#elif SIZEOF_VOIDP == SIZEOF_LONG_LONG
-#define MSB(i) ((int)sizeof(long long)*8-1-__builtin_clzll(i)) /* GCC builtin */
-#define HAVE_MSB 1
-#endif
 #define HAVE_MSB64 1
-#define MSB64(i) ((int)sizeof(long long)*8-1-__builtin_clzll(i))
+
+static inline unsigned int
+MSB(size_t i)
+{
+#if SIZEOF_VOIDP == SIZEOF_LONG
+  return (unsigned int)sizeof(long)*8-1-__builtin_clzl(i);
+#elif SIZEOF_VOIDP == SIZEOF_LONG_LONG
+  return (unsigned int)sizeof(long long)*8-1-__builtin_clzll(i);
+#else
+#error "No MSB";
+#endif
+}
+
+static inline unsigned int
+MSB64(int64_t i)
+{ return (unsigned int)sizeof(long long)*8-1-__builtin_clzll(i);
+}
 #endif
 
 #ifdef HAVE_GCC_ATOMIC
@@ -273,6 +283,20 @@ COMPARE_AND_SWAP_WORD(word *at, word from, word to)
 #endif
 }
 
+static inline int
+COMPARE_AND_SWAP_ATOM(atom_t *at, atom_t from, atom_t to)
+{
+#ifdef _MSC_VER
+# if SIZEOF_ATOM == 4
+  return _InterlockedCompareExchange(at, to, from) == from;
+# else
+  return _InterlockedCompareExchange64(at, to, from) == from;
+#endif
+#else
+  return __COMPARE_AND_SWAP(at, from, to);
+#endif
+}
+
 #else
 #define ATOMIC_ADD(ptr, v)		(*ptr += v)
 #define ATOMIC_SUB(ptr, v)		(*ptr -= v)
@@ -288,6 +312,7 @@ COMPARE_AND_SWAP_WORD(word *at, word from, word to)
 #define COMPARE_AND_SWAP_UINT(ptr,o,n)	COMPARE_AND_SWAP(ptr,o,n)
 #define COMPARE_AND_SWAP_SIZE(ptr,o,n)	COMPARE_AND_SWAP(ptr,o,n)
 #define COMPARE_AND_SWAP_WORD(ptr,o,n)	COMPARE_AND_SWAP(ptr,o,n)
+#define COMPARE_AND_SWAP_ATOM(ptr,o,n)	COMPARE_AND_SWAP(ptr,o,n)
 #endif
 
 #ifndef HAVE_MSB
@@ -391,18 +416,14 @@ typedef struct bit_vector
 } bit_vector;
 #define BITSPERE (sizeof(bitv_chunk)*8)
 
-#ifndef offset
-#define offset(s, f) ((size_t)(&((struct s *)NULL)->f))
-#endif
-
 static inline size_t
 sizeof_bitvector(size_t bits)
-{ return offset(bit_vector, chunk[(bits+BITSPERE-1)/BITSPERE]);
+{ return offsetof(struct bit_vector, chunk[(bits+BITSPERE-1)/BITSPERE]);
 }
 
 static inline void
 init_bitvector(bit_vector *v, size_t bits)
-{ size_t bytes = offset(bit_vector, chunk[(bits+BITSPERE-1)/BITSPERE]);
+{ size_t bytes = offsetof(struct bit_vector, chunk[(bits+BITSPERE-1)/BITSPERE]);
 
   memset(v, 0, bytes);
   v->size = bits;
@@ -410,7 +431,7 @@ init_bitvector(bit_vector *v, size_t bits)
 
 static inline bit_vector *
 new_bitvector(size_t size)
-{ size_t bytes = offset(bit_vector, chunk[(size+BITSPERE-1)/BITSPERE]);
+{ size_t bytes = offsetof(struct bit_vector, chunk[(size+BITSPERE-1)/BITSPERE]);
   bit_vector *v = allocHeapOrHalt(bytes);
 
   memset(v, 0, bytes);
@@ -420,7 +441,7 @@ new_bitvector(size_t size)
 
 static inline void
 free_bitvector(bit_vector *v)
-{ size_t bytes = offset(bit_vector, chunk[(v->size+BITSPERE-1)/BITSPERE]);
+{ size_t bytes = offsetof(struct bit_vector, chunk[(v->size+BITSPERE-1)/BITSPERE]);
 
   freeHeap(v, bytes);
 }
@@ -507,18 +528,17 @@ Trail(DECL_LD Word p, word v)
 }
 
 
-#define consPtrB(p, base, ts)	f_consPtr(p, (uintptr_t)(base), ts)
-#define consPtr(p, ts)		consPtrB(p, LD->bases[(ts)&STG_MASK], (ts))
-#define f_consPtr(p, base, ts) LDFUNC(f_consPtr, p, base, ts)
 static inline word
-f_consPtr(DECL_LD void *p, uintptr_t base, word ts)
-{ uintptr_t v = (uintptr_t) p;
+consPtr(void *p, word ts)
+{ word v = ptr2word(p);
 
-  v -= base;
-  DEBUG(CHK_SECURE, assert(v < MAXTAGGEDPTR && !(v&0x3)));
-  return (v<<5)|ts;
+  return (v<<LMASK_BITS)|ts;
 }
 
+static inline Word
+valPtr(word w)
+{ return word2ptr(Word, w>>LMASK_BITS);
+}
 
 #if ALIGNOF_DOUBLE == ALIGNOF_VOIDP
 #define valFloat(w) (*(double *)valIndirectP(w))
@@ -535,30 +555,6 @@ valFloat(DECL_LD word w)
 #endif
 
 
-#if ALIGNOF_INT64_T == ALIGNOF_VOIDP
-#define valBignum(w) (*(int64_t *)valIndirectP(w))
-#else
-#define valBignum(w) LDFUNC(valBignum, w)
-static inline int64_t
-valBignum(DECL_LD word w)
-{ Word p = valIndirectP(w);
-  union
-  { int64_t i;
-    word w[WORDS_PER_INT64];
-  } val;
-
-#if ( SIZEOF_VOIDP == 4 )
-  val.w[0] = p[0];
-  val.w[1] = p[1];
-#else
-#error "Unsupported int64_t alignment conversion"
-#endif
-
-  return val.i;
-}
-#endif
-
-
 #define valHandle(r) LDFUNC(valHandle, r)
 static inline word
 valHandle(DECL_LD term_t r)
@@ -576,15 +572,12 @@ checking (unless compiled for debugging) and fetches the base address of
 the global stack only once.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
-#define linkValI(p) LDFUNC(linkValI, p)
 static inline word
-linkValI(DECL_LD Word p)
+linkValI(Word p)
 { word w = *p;
-  uintptr_t gb = LD->bases[STG_GLOBAL];
 
   while(isRef(w))
-  { //p = unRef(w);
-    p = (Word)valPtrB(w,gb);
+  { p = unRef(w);
     w = *p;
   }
 
@@ -593,9 +586,8 @@ linkValI(DECL_LD Word p)
   if ( !needsRef(w) )
   { return w;
   } else
-  { // return makeRefG(p);
-    DEBUG(0, assert(p<(Word)lBase));
-    return consPtrB(p, gb, TAG_REFERENCE|STG_GLOBAL);
+  { DEBUG(0, assert(p<(Word)lBase));
+    return makeRefG(p);
   }
 }
 
@@ -646,7 +638,7 @@ visibleClause(DECL_LD Clause cl, gen_t gen)
     return TRUE;
 
   if ( unlikely(LD->transaction.gen_base && gen >= LD->transaction.gen_base) &&
-       true(cl->predicate, P_DYNAMIC) )
+       true(cl->predicate, P_TRANSACT) )
     return transaction_visible_clause(cl, gen);
 
   return FALSE;
@@ -669,7 +661,7 @@ global_generation(void)
 #define current_generation(def) LDFUNC(current_generation, def)
 static inline gen_t
 current_generation(DECL_LD Definition def)
-{ if ( unlikely(!!LD->transaction.generation) && def && true(def, P_DYNAMIC) )
+{ if ( unlikely(!!LD->transaction.generation) && def && true(def, P_TRANSACT) )
   { return LD->transaction.generation;
   } else
   { return GD->_generation;
@@ -679,7 +671,7 @@ current_generation(DECL_LD Definition def)
 #define next_generation(def) LDFUNC(next_generation, def)
 static inline gen_t
 next_generation(DECL_LD Definition def)
-{ if ( unlikely(!!LD->transaction.generation) && def && true(def, P_DYNAMIC) )
+{ if ( unlikely(!!LD->transaction.generation) && def && true(def, P_TRANSACT) )
   { if ( LD->transaction.generation < LD->transaction.gen_max )
       return ++LD->transaction.generation;
     return 0;
@@ -697,7 +689,7 @@ next_generation(DECL_LD Definition def)
 #define max_generation(def) LDFUNC(max_generation, def)
 static inline gen_t
 max_generation(DECL_LD Definition def)
-{ if ( unlikely(!!LD->transaction.generation) && def && true(def, P_DYNAMIC) )
+{ if ( unlikely(!!LD->transaction.generation) && def && true(def, P_TRANSACT) )
     return LD->transaction.gen_max;
   else
     return GEN_MAX;
@@ -718,7 +710,7 @@ generation is updated and thus no harm is done.
 static inline void
 setGenerationFrame(DECL_LD LocalFrame fr)
 { if ( unlikely(LD->transaction.generation &&
-		true(fr->predicate, P_DYNAMIC)) )
+		true(fr->predicate, P_TRANSACT)) )
   { setGenerationFrameVal(fr, LD->transaction.generation);
   } else
   { gen_t gen;
