@@ -3,7 +3,7 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  1995-2022, University of Amsterdam
+    Copyright (c)  1995-2024, University of Amsterdam
 			      CWI, Amsterdam
 			      SWI-Prolog Solutions b.v.
     All rights reserved.
@@ -213,6 +213,11 @@ int
 Pause(double t)
 { HANDLE h;
 
+  if ( t <= 0.0 )
+  { SwitchToThread();
+    return TRUE;
+  }
+
   if ( (h = CreateWaitableTimer(NULL, TRUE, NULL)) )
   { LARGE_INTEGER ft;
 
@@ -304,10 +309,10 @@ CpuTime(cputime_kind which)
 	break;
       case CPU_SYSTEM:
 	p = &kerneltime;
-        break;
+	break;
       default:
 	assert(0);
-        return 0.0;
+	return 0.0;
     }
     t = (double)p->dwHighDateTime * (4294967296.0 * ntick nano);
     t += (double)p->dwLowDateTime  * (ntick nano);
@@ -331,10 +336,18 @@ CpuCount(void)
 
 void
 setOSPrologFlags(void)
-{ PL_set_prolog_flag("cpu_count", PL_INTEGER, CpuCount());
+{ PL_set_prolog_flag("cpu_count", PL_INTEGER, (intptr_t)CpuCount());
+#ifdef MSYS2
+  PL_set_prolog_flag("msys2", PL_BOOL|FF_READONLY, TRUE);
+#endif
 }
 
 
+/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+findExecutable() returns  a Prolog path  for the main executable  or a
+module of SWI-Prolog.  Returns NULL if the path cannot be determined.
+- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
+
 char *
 findExecutable(const char *module, char *exe, size_t exelen)
 { int n;
@@ -354,14 +367,18 @@ findExecutable(const char *module, char *exe, size_t exelen)
     hmod = NULL;
 
   if ( (n = GetModuleFileNameW(hmod, wbuf, PATH_MAX)) > 0 )
-  { wbuf[n] = EOS;
-    return _xos_long_file_name_toA(wbuf, exe, exelen);
+  { char os_exe[PATH_MAX];
+    char *p0;
+
+    wbuf[n] = EOS;
+    if ( (p0=_xos_long_file_name_toA(wbuf, os_exe, sizeof(os_exe))) )
+      return PrologPath(p0, exe, exelen);
+    return NULL;
   } else if ( module )
   { return PrologPath(module, exe, exelen);
   } else
-    *exe = EOS;
-
-  return exe;
+  { return NULL;
+  }
 }
 
 char *
@@ -548,7 +565,7 @@ System(char *command)			/* command is a UTF-8 string */
 }
 
 
-word
+foreign_t
 pl_win_exec(term_t cmd, term_t how)
 { wchar_t *s;
   size_t len;
@@ -733,7 +750,7 @@ PRED_IMPL("win_add_dll_directory", 2, win_add_dll_directory, 0)
       /* AddDllDirectoryW() cannot handle "\\?\" */
       if ( (cookie = (*f_AddDllDirectoryW)(dirw + _xos_win_prefix_length(dirw))) )
       { DEBUG(MSG_WIN_API,
-	      Sdprintf("AddDllDirectory(%Ws) ok\n", dirw));
+	      SdprintfX("AddDllDirectory(%Ws) ok\n", dirw));
 
 	return PL_unify_int64(A2, (int64_t)(uintptr_t)cookie);
       }
@@ -741,7 +758,7 @@ PRED_IMPL("win_add_dll_directory", 2, win_add_dll_directory, 0)
       switch((eno=GetLastError()))
       { case ERROR_FILE_NOT_FOUND:
 	  return PL_existence_error("directory", A1);
-        case ERROR_INVALID_PARAMETER:
+	case ERROR_INVALID_PARAMETER:
 	  return PL_domain_error("absolute_file_name", A1);
       }
       return PL_error(NULL, 0, WinErrorNo(eno), ERR_SYSCALL, "AddDllDirectory()");
@@ -807,7 +824,7 @@ PL_dlopen(const char *file, int flags)	/* file is in UTF-8, POSIX path */
     *w = 0;
   }
 
-  DEBUG(MSG_WIN_API, Sdprintf("dlopen(%Ws)\n", wfile));
+  DEBUG(MSG_WIN_API, SdprintfX("dlopen(%Ws)\n", wfile));
 
   if ( is_windows_abs_path(wfile) )
     llflags |= load_library_search_flags();
@@ -1180,7 +1197,7 @@ PRED_IMPL("win_registry_get_value", 3, win_registry_get_value, 0)
 	return PL_unify_integer(Value, data.dword);
       default:
 	warning("get_registry_value/2: Unknown registery-type: %d", type);
-        fail;
+	fail;
     }
   }
 
@@ -1259,7 +1276,7 @@ PL_w32_running_under_wine(void)
 
 
 		 /*******************************
-		 *	        MUI		*
+		 *		MUI		*
 		 *******************************/
 
 
@@ -1357,4 +1374,3 @@ BeginPredDefs(win)
 EndPredDefs
 
 #endif /*__WINDOWS__*/
-
