@@ -379,6 +379,12 @@ zseek64_mem(voidpf opaque, voidpf stream, ZPOS64_T offset, int origin)
 { mem_stream *mem = stream;
   const char *p;
 
+  /* Avoid UBSAN in start + offset */
+  if ( origin == SEEK_SET && offset == (ZPOS64_T)(-1) )
+  { errno = EINVAL;
+    return -1;
+  }
+ 
   switch(origin)
   { case SEEK_SET: p = mem->start + offset; break;
     case SEEK_CUR: p = mem->here + offset;  break;
@@ -580,7 +586,7 @@ get_zipper(term_t t, zipper **zipper)
 /** zip_open_stream(+Stream, -Zipper, +Options)
 */
 
-static const opt_spec zip_open_stream_options[] =
+static const PL_option_t zip_open_stream_options[] =
 { { ATOM_close_parent,	    OPT_BOOL },
   { NULL_ATOM,		    0 }
 };
@@ -591,7 +597,7 @@ PRED_IMPL("zip_open_stream", 3, zip_open_stream, 0)
   IOSTREAM *stream = NULL;
   int close_parent = FALSE;
 
-  if ( !scan_options(A3, 0, ATOM_zip_options, zip_open_stream_options,
+  if ( !PL_scan_options(A3, 0, "zip_options", zip_open_stream_options,
 		     &close_parent) )
     return FALSE;
 
@@ -658,7 +664,8 @@ PRED_IMPL("zip_clone", 2, zip_clone, 0)
 #endif
     if ( clone->path )
       clone->path = strdup(clone->path);
-    clone->reader     = unzClone(clone->reader);
+    clone->reader = unzClone(clone->reader);
+    clone->mapped_file = NULL;			/* I'm just a clone */
 
     return unify_zipper(A2, clone);
   }
@@ -670,7 +677,7 @@ PRED_IMPL("zip_clone", 2, zip_clone, 0)
 */
 
 static
-PRED_IMPL("zip_close", 2, zip_close, 0)
+PRED_IMPL("zip_close_", 2, zip_close, 0)
 { PRED_LD
   char *comment = NULL;
   zipper *z;
@@ -681,11 +688,13 @@ PRED_IMPL("zip_close", 2, zip_close, 0)
        (PL_is_variable(A2) || PL_get_chars(A2, &comment, flags)) &&
        zacquire(z, ZIP_CLOSE, &prev_state, "close") )
   { if ( prev_state == ZIP_READ_ENTRY )
+    { PL_register_atom(z->symbol);
       return TRUE;				/* delay */
-    else if ( close_zipper(z) == 0 )
-      return TRUE;
-    else
-      return PL_warning("zip_close/2 failed");
+    } else if ( close_zipper(z) == 0 )
+    { return TRUE;
+    } else
+    { return PL_warning("zip_close/2 failed");
+    }
   }
 
   return FALSE;
@@ -745,6 +754,16 @@ Swrite_zip_entry(void *handle, char *buf, size_t size)
   }
 }
 
+static long
+Sseek_zip_entry(void *handle, long pos, int whence)
+{ (void) handle;
+  (void) pos;
+  (void) whence;
+
+  errno = ESPIPE;
+  return -1;
+}
+
 static int
 Sclose_zip_entry(void *handle)
 { zipper *z = handle;
@@ -758,6 +777,7 @@ Sclose_zip_entry(void *handle)
   if ( z->state == ZIP_CLOSE )
   { zrelease(z);
     close_zipper(z);
+    PL_register_atom(z->symbol);		/* revert delayed zip_close_/2 */
   } else if ( true(z, ZIP_RELEASE_ON_CLOSE) )
   { zrelease(z);
   } else
@@ -800,12 +820,20 @@ Scontrol_zip_entry(void *handle, int action, void *arg)
 IOFUNCTIONS Szipfunctions =
 { Sread_zip_entry,
   Swrite_zip_entry,
-  NULL,						/* seek */
+  NULL,
   Sclose_zip_entry,
-  Scontrol_zip_entry,				/* control */
+  Scontrol_zip_entry,
   NULL						/* seek64 */
 };
 
+IOFUNCTIONS Szipfunctions_repositioning =
+{ Sread_zip_entry,
+  Swrite_zip_entry,
+  Sseek_zip_entry,
+  Sclose_zip_entry,
+  Scontrol_zip_entry,
+  NULL						/* seek64 */
+};
 
 		 /*******************************
 		 *	  HANDLE ENTRIES	*
@@ -815,7 +843,7 @@ static void
 zset_time(zip_fileinfo *info, double t)
 { struct tm rb, *r;
   tm_zip *tmzip = &info->tmz_date;
-  time_t itime = t;
+  time_t itime = (time_t)t;
 
   r = PL_localtime_r(&itime, &rb);
   tmzip->tm_sec  = r->tm_sec;
@@ -846,7 +874,7 @@ zget_time(const unz_file_info64 *info)
 /** zipper_open_new_file_in_zip(+Zipper, +Name, -Stream, +Options)
 */
 
-static const opt_spec zip_new_file_options[] =
+static const PL_option_t zip_new_file_options[] =
 { { ATOM_extra,		    OPT_STRING },
   { ATOM_comment,	    OPT_STRING },
   { ATOM_time,		    OPT_DOUBLE },
@@ -872,7 +900,7 @@ PRED_IMPL("zipper_open_new_file_in_zip", 4, zipper_open_new_file_in_zip, 0)
   int imethod;
   int zip64 = FALSE;
 
-  if ( !scan_options(A4, 0, ATOM_zip_options, zip_new_file_options,
+  if ( !PL_scan_options(A4, 0, "zip_options", zip_new_file_options,
 		     &extra, &comment, &ftime, &method, &level, &zip64) )
     return FALSE;
 
@@ -1019,27 +1047,30 @@ PRED_IMPL("zipper_goto", 2, zipper_goto, 0)
  *  Open the current file as an input stream
  */
 
-static const opt_spec zipopen3_options[] =
+static const PL_option_t zipopen3_options[] =
 { { ATOM_type,		 OPT_ATOM },
   { ATOM_encoding,	 OPT_ATOM },
   { ATOM_bom,		 OPT_BOOL },
   { ATOM_release,	 OPT_BOOL },
-  { NULL_ATOM,	         0 }
+  { ATOM_reposition,	 OPT_BOOL },
+  { NULL_ATOM,		 0 }
 };
 
 static
 PRED_IMPL("zipper_open_current", 3, zipper_open_current, 0)
 { PRED_LD
   zipper *z;
-  atom_t type     = ATOM_text;
-  atom_t encoding = NULL_ATOM;
-  int	 bom      = -1;
-  int    release  = TRUE;
-  int flags       = SIO_INPUT|SIO_RECORDPOS|SIO_FBUF;
+  atom_t type       = ATOM_text;
+  atom_t encoding   = NULL_ATOM;
+  int	 bom        = -1;
+  int    release    = TRUE;
+  int	 reposition = FALSE;
+  size_t size       = 0;
+  int flags         = SIO_INPUT|SIO_RECORDPOS|SIO_FBUF;
   IOENC enc;
 
-  if ( !scan_options(A3, 0, ATOM_stream_option, zipopen3_options,
-		     &type, &encoding, &bom, &release) )
+  if ( !PL_scan_options(A3, 0, "stream_option", zipopen3_options,
+		     &type, &encoding, &bom, &release, &reposition) )
     return FALSE;
   if ( !stream_encoding_options(type, encoding, &bom, &enc) )
     return FALSE;
@@ -1063,12 +1094,37 @@ PRED_IMPL("zipper_open_current", 3, zipper_open_current, 0)
 
     if ( release )
       set(z, ZIP_RELEASE_ON_CLOSE);
+
+    if ( reposition )
+    { unz_file_info64 info;
+      char fname[PATH_MAX];
+      char extra[1024];
+      char comment[1024];
+
+      if ( unzGetCurrentFileInfo64(z->reader,
+				 &info,
+				 fname, sizeof(fname),
+				 extra, sizeof(extra),
+				 comment, sizeof(comment)) == UNZ_OK )
+      {
+#if SIZEOF_VOIDP < 8
+	if ( info.uncompressed_size > SIZE_MAX )
+	  PL_representation_error("size_t");
+#endif
+	size = (size_t)info.uncompressed_size;
+      }
+    }
+
     if ( unzOpenCurrentFile(z->reader) == UNZ_OK )
-    { IOSTREAM *s = Snew(z, flags, &Szipfunctions);
+    { IOSTREAM *s = Snew(z, flags, reposition ? &Szipfunctions_repositioning
+					      : &Szipfunctions);
 
       if ( s )
       { s->encoding = enc;
 
+	if ( reposition && size )
+	  Ssetbuffer(s, NULL, size);
+
 	if ( bom && ScheckBOM(s) < 0 )
 	  return PL_release_stream(PL_acquire_stream(s));
 
@@ -1090,7 +1146,7 @@ PRED_IMPL("zip_file_info_", 3, zip_file_info, 0)
 
   if ( get_zipper(A1, &z) )
   { unz_file_info64 info;
-    char fname[MAXPATHLEN];
+    char fname[PATH_MAX];
     char extra[1024];
     char comment[1024];
 
@@ -1164,8 +1220,17 @@ map_file(const char *name)
   { struct stat buf;
 
     if ( fstat(fd, &buf) == 0 )
-    { mf->start = mmap(NULL,
-		       buf.st_size,
+    {
+#if SIZEOF_VOIDP < 8
+      if ( buf.st_size > SIZE_MAX )
+      { close(fd);
+	free(mf);
+	PL_representation_error("size_t");
+	return NULL;
+      }
+#endif
+      mf->start = mmap(NULL,
+		       (size_t)buf.st_size,
 		       PROT_READ,
 		       MAP_SHARED,
 		       fd,
@@ -1411,14 +1476,14 @@ zipper_file(const zipper *z)
 		 *******************************/
 
 BeginPredDefs(zip)
-  PRED_DEF("zip_open_stream",		  3, zip_open_stream,	          0)
-  PRED_DEF("zip_clone",		          2, zip_clone,	                  0)
-  PRED_DEF("zip_close_",		  2, zip_close,		          0)
-  PRED_DEF("zip_lock",		          1, zip_lock,		          0)
-  PRED_DEF("zip_unlock",	          1, zip_unlock,		  0)
+  PRED_DEF("zip_open_stream",		  3, zip_open_stream,		  0)
+  PRED_DEF("zip_clone",			  2, zip_clone,			  0)
+  PRED_DEF("zip_close_",		  2, zip_close,			  0)
+  PRED_DEF("zip_lock",			  1, zip_lock,			  0)
+  PRED_DEF("zip_unlock",		  1, zip_unlock,		  0)
   PRED_DEF("zipper_open_new_file_in_zip", 4, zipper_open_new_file_in_zip, 0)
   PRED_DEF("zipper_goto",		  2, zipper_goto,		  0)
   PRED_DEF("zipper_open_current",         3, zipper_open_current,         0)
   PRED_DEF("zip_file_info_",		  3, zip_file_info,               0)
-  PRED_DEF("$rc_handle",		  1, rc_handle,		          0)
+  PRED_DEF("$rc_handle",		  1, rc_handle,			  0)
 EndPredDefs
